@@ -3,7 +3,7 @@
 // Purpose:     Canvas items
 // Author:      Alex Thuering
 // Created:     2005/05/09
-// RCS-ID:      $Id: SVGCanvasItem.h,v 1.19 2011/08/08 06:53:36 ntalex Exp $
+// RCS-ID:      $Id: SVGCanvasItem.h,v 1.21 2011/11/23 22:14:53 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -147,13 +147,14 @@ class wxSVGCanvasText: public wxSVGCanvasItem
 /** Canvas item, that saves image (SVGImageElement) */
 class wxSVGCanvasImage: public wxSVGCanvasItem {
 public:
-	wxSVGCanvasImage(): wxSVGCanvasItem(wxSVG_CANVAS_ITEM_IMAGE) {}
-	wxSVGCanvasImage(wxSVGCanvasItemType type): wxSVGCanvasItem(type) {}
-	virtual ~wxSVGCanvasImage() {}
+	wxSVGCanvasImage(): wxSVGCanvasItem(wxSVG_CANVAS_ITEM_IMAGE), m_svgImage(NULL) {}
+	wxSVGCanvasImage(wxSVGCanvasItemType type): wxSVGCanvasItem(type), m_svgImage(NULL) {}
+	virtual ~wxSVGCanvasImage();
 	virtual void Init(wxSVGImageElement& element, const wxCSSStyleDeclaration& style);
-	virtual int GetDefaultWidth() { return m_image.Ok() ? m_image.GetWidth() : 0; }
-	virtual int GetDefaultHeight() { return m_image.Ok() ? m_image.GetHeight() * m_defHeightScale : 0; }
+	virtual int GetDefaultWidth();
+	virtual int GetDefaultHeight();
 	const wxSVGPreserveAspectRatio& GetPreserveAspectRatio() { return m_preserveAspectRatio; }
+	wxSVGSVGElement* GetSvgImage() { return m_svgImage; }
 	
 public:
 	double m_x, m_y, m_width, m_height; /** position and size of image */
@@ -161,26 +162,39 @@ public:
 	wxImage m_image; /** image data */
 	double m_defHeightScale;
 	wxSVGPreserveAspectRatio m_preserveAspectRatio;
+	wxSVGSVGElement* m_svgImage;
+};
+
+class wxFfmpegMediaDecoder;
+
+/** CanvasVideoData */
+class wxSVGCanvasVideoData {
+public:
+	wxSVGCanvasVideoData(wxFfmpegMediaDecoder* mediaDecoder);
+	~wxSVGCanvasVideoData();
+	
+	void IncRef() { m_count++; }
+	int DecRef() { return (--m_count); }
+	
+	wxFfmpegMediaDecoder* GetMediaDecoder() { return m_mediaDecoder; }
+	
+private:
+	int m_count;
+	wxFfmpegMediaDecoder* m_mediaDecoder;
 };
 
 /** Canvas item, that saves video (wxSVGVideoElement) */
-#ifdef USE_FFMPEG
-class wxFfmpegMediaDecoder;
-#endif
-class wxSVGCanvasVideo: public wxSVGCanvasImage
-{
-  public:
+class wxSVGCanvasVideo: public wxSVGCanvasImage {
+public:
 	wxSVGCanvasVideo();
 	virtual ~wxSVGCanvasVideo();
 	virtual void Init(wxSVGVideoElement& element, const wxCSSStyleDeclaration& style);
-    double GetDuration() { return m_duration; }
-  
-  public:
+	double GetDuration() { return m_duration; }
+	
+public:
 	double m_time; /** time of the loaded frame */
 	double m_duration;
-#ifdef USE_FFMPEG
-	wxFfmpegMediaDecoder* m_mediaDecoder;
-#endif
+	wxSVGCanvasVideoData* m_videoData;
 };
 
 #endif // WX_SVG_CANVAS_ITEM_H
