@@ -3,7 +3,7 @@
 // Purpose:     
 // Author:      Alex Thuering
 // Created:     2005/05/09
-// RCS-ID:      $Id: SVGCanvasItem.cpp,v 1.33 2011/08/08 06:53:14 ntalex Exp $
+// RCS-ID:      $Id: SVGCanvasItem.cpp,v 1.38 2011/11/24 00:02:55 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 //////////////////////////////////////////////////////////////////////////////
@@ -567,11 +567,19 @@ void wxSVGCanvasText::InitChildren(wxSVGTextPositioningElement& element, const w
 	wxSVGElement* elem = (wxSVGElement*) element.GetChildren();
 	while (elem) {
 		if (elem->GetType() == wxSVGXML_TEXT_NODE) {
-			wxStringTokenizer tokenizer(elem->GetContent());
-			while (tokenizer.HasMoreTokens()) {
-				text += tokenizer.GetNextToken();
-				if (tokenizer.HasMoreTokens())
-					text += wxT(" ");
+			if (element.GetXmlspace() == wxT("preserve")) {
+				wxString t = elem->GetContent();
+				for (unsigned int i = 0; i< t.length(); i++)
+					if (t[i] == wxT('\t') || t[i] == wxT('\n') || t[i] == wxT('\r'))
+						t[i] = wxT('\t');
+				text += t;
+			} else {
+				wxStringTokenizer tokenizer(elem->GetContent());
+				while (tokenizer.HasMoreTokens()) {
+					text += tokenizer.GetNextToken();
+					if (tokenizer.HasMoreTokens())
+						text += wxT(" ");
+				}
 			}
 		} else if (elem->GetType() == wxSVGXML_ELEMENT_NODE && elem->GetDtd() == wxSVG_TBREAK_ELEMENT)
 			text += wxT("\n");
@@ -843,6 +851,10 @@ double wxSVGCanvasText::GetRotationOfChar(unsigned long charnum)
 //////////////////////////////////////////////////////////////////////////////
 ////////////////////////////// wxSVGCanvasImage //////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
+wxSVGCanvasImage::~wxSVGCanvasImage() {
+	if (m_svgImage)
+		delete m_svgImage;
+}
 
 void wxSVGCanvasImage::Init(wxSVGImageElement& element, const wxCSSStyleDeclaration& style) {
 	m_x = element.GetX().GetAnimVal();
@@ -856,17 +868,29 @@ void wxSVGCanvasImage::Init(wxSVGImageElement& element, const wxCSSStyleDeclarat
 	if (prevItem != NULL && prevItem->m_href == m_href) {
 		m_image = prevItem->m_image;
 		m_defHeightScale = prevItem->m_defHeightScale;
+		m_svgImage = prevItem->m_svgImage != NULL ? new wxSVGSVGElement(*prevItem->m_svgImage) : NULL;
 	} else if (m_href.length()) {
 		long pos = 0;
 		wxString filename = m_href;
-		if (m_href.Find(wxT('#')) != wxNOT_FOUND) {
-			filename = m_href.Before(wxT('#'));
-			m_href.After(wxT('#')).ToLong(&pos);
-		}
+		if (!wxFileExists(filename) && m_href.Find(wxT('#')) != wxNOT_FOUND && m_href.AfterLast(wxT('#')).ToLong(&pos))
+			filename = m_href.BeforeLast(wxT('#'));
 		if (!wxFileExists(filename)) {
 			wxLogError(_("Can't load image from file '%s': file does not exist."), filename.c_str());
 			return;
 		}
+		if (element.GetHref().GetAnimVal().EndsWith(wxT(".svg"))) {
+			wxSVGDocument imgDoc;
+			if (imgDoc.Load(filename) && imgDoc.GetRootElement() != NULL) {
+				m_svgImage = imgDoc.GetRootElement();
+				imgDoc.RemoveChild(m_svgImage);
+				if (m_svgImage->GetViewBox().GetBaseVal().IsEmpty()
+						&& m_svgImage->GetWidth().GetBaseVal().GetValue() > 0
+						&& m_svgImage->GetWidth().GetBaseVal().GetUnitType() != wxSVG_LENGTHTYPE_PERCENTAGE)
+					m_svgImage->SetViewBox(
+							wxSVGRect(0, 0, m_svgImage->GetWidth().GetBaseVal(), m_svgImage->GetHeight().GetBaseVal()));
+			}
+			return;
+		}
 #ifdef USE_FFMPEG
 		bool log = wxLog::EnableLogging(false);
 		m_image.LoadFile(filename);
@@ -905,24 +929,42 @@ void wxSVGCanvasImage::Init(wxSVGImageElement& element, const wxCSSStyleDeclarat
 	}
 }
 
+int wxSVGCanvasImage::GetDefaultWidth() {
+	if (GetSvgImage() != NULL && !GetSvgImage()->GetViewBox().GetBaseVal().IsEmpty())
+		return GetSvgImage()->GetViewBox().GetBaseVal().GetWidth();
+	return  m_image.Ok() ? m_image.GetWidth() : 0;
+}
+
+int wxSVGCanvasImage::GetDefaultHeight() {
+	if (GetSvgImage() != NULL && !GetSvgImage()->GetViewBox().GetBaseVal().IsEmpty())
+		return GetSvgImage()->GetViewBox().GetBaseVal().GetHeight();
+	return m_image.Ok() ? m_image.GetHeight() * m_defHeightScale : 0;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 ////////////////////////////// wxSVGCanvasVideo //////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
-
-wxSVGCanvasVideo::wxSVGCanvasVideo(): wxSVGCanvasImage(wxSVG_CANVAS_ITEM_VIDEO)
-{
-#ifdef USE_FFMPEG
-  m_mediaDecoder = NULL;
-#endif
+wxSVGCanvasVideoData::wxSVGCanvasVideoData(wxFfmpegMediaDecoder* mediaDecoder) {
+	m_count = 1;
+	m_mediaDecoder = mediaDecoder;
 }
 
-wxSVGCanvasVideo::~wxSVGCanvasVideo()
-{
+wxSVGCanvasVideoData::~wxSVGCanvasVideoData() {
 #ifdef USE_FFMPEG
-  if (m_mediaDecoder)
-    delete m_mediaDecoder;
+	if (m_mediaDecoder)
+		delete m_mediaDecoder;
 #endif
 }
+	
+wxSVGCanvasVideo::wxSVGCanvasVideo(): wxSVGCanvasImage(wxSVG_CANVAS_ITEM_VIDEO), m_videoData(NULL) {
+	// nothing to do
+}
+
+wxSVGCanvasVideo::~wxSVGCanvasVideo() {
+	if (m_videoData != NULL && m_videoData->DecRef() == 0)
+		delete m_videoData;
+}
 
 void wxSVGCanvasVideo::Init(wxSVGVideoElement& element, const wxCSSStyleDeclaration& style) {
 	m_x = element.GetX().GetAnimVal();
@@ -931,59 +973,60 @@ void wxSVGCanvasVideo::Init(wxSVGVideoElement& element, const wxCSSStyleDeclarat
 	m_height = element.GetHeight().GetAnimVal();
 	m_href = element.GetHref();
 	m_preserveAspectRatio = element.GetPreserveAspectRatio();
-	m_time = ((wxSVGDocument*) element.GetOwnerDocument())->GetCurrentTime();
+	m_time = element.GetOwnerDocument() != NULL ? ((wxSVGDocument*) element.GetOwnerDocument())->GetCurrentTime() : 0;
+	if (element.GetBegin() > 0)
+		m_time = m_time > element.GetBegin() ? m_time - element.GetBegin() : 0;
+	double dur = element.GetDur() > 0 ? element.GetDur() : element.GetClipEnd() - element.GetClipBegin();
+	if (dur > 0 && m_time > dur)
+		m_time = dur;
+	m_time += element.GetClipBegin();
+#ifdef USE_FFMPEG
 	wxSVGCanvasVideo* prevItem = (wxSVGCanvasVideo*) element.GetCanvasItem();
-	if (prevItem != NULL && prevItem->m_href == m_href
-#ifdef USE_FFMPEG
-			&& prevItem->m_mediaDecoder != NULL
-#endif
-			) {
-#ifdef USE_FFMPEG
-		m_mediaDecoder = prevItem->m_mediaDecoder;
-		prevItem->m_mediaDecoder = NULL;
-#endif
+	if (prevItem != NULL && prevItem->m_href == m_href && prevItem->m_videoData != NULL) {
+		m_videoData = prevItem->m_videoData;
+		m_videoData->IncRef();
 		m_duration = prevItem->m_duration;
-#ifdef USE_FFMPEG
-		double ftime = m_mediaDecoder->GetFps() != -1 ? 1.0 / m_mediaDecoder->GetFps() : 0.04;
-		double currTime = m_mediaDecoder->GetPosition();
-		if (currTime != m_time && (m_time < currTime || m_time - currTime >= ftime - 0.001)) {
-			if (m_time < currTime || m_time - currTime > 50*ftime)
-				m_mediaDecoder->SetPosition(m_time > 1.0 ? m_time - 1.0 : 0.0);
-			for (int i = 0; i < 60; i++) {
-				m_image = m_mediaDecoder->GetNextFrame();
-				currTime = m_mediaDecoder->GetPosition();
-				if (currTime >= m_time - 0.001 || currTime < 0)
-					break;
-			}
-		} else
-#endif
-			m_image = prevItem->m_image;
+		wxFfmpegMediaDecoder* decoder = m_videoData->GetMediaDecoder();
+		if (decoder != NULL) {
+			double ftime = decoder->GetFps() != -1 ? 1.0 / decoder->GetFps() : 0.04;
+			double currTime = decoder->GetPosition();
+			if (currTime != m_time && (m_time < currTime || m_time - currTime >= ftime - 0.001)) {
+				if (m_time < currTime || m_time - currTime > 50*ftime)
+					decoder->SetPosition(m_time > 1.0 ? m_time - 1.0 : 0.0);
+				for (int i = 0; i < 60; i++) {
+					m_image = decoder->GetNextFrame();
+					currTime = decoder->GetPosition();
+					if (currTime >= m_time - 0.001 || currTime < 0)
+						break;
+				}
+			} else
+				m_image = prevItem->m_image;
+		}
 	} else if (m_href.length()) {
-#ifdef USE_FFMPEG
-		m_mediaDecoder = new wxFfmpegMediaDecoder();
-		if (m_mediaDecoder->Load(m_href)) {
-			m_duration = m_mediaDecoder->GetDuration();
+		wxFfmpegMediaDecoder* decoder = new wxFfmpegMediaDecoder();
+		if (decoder->Load(m_href)) {
+			m_duration = decoder->GetDuration();
 			if (m_time > 0) {
-				m_image = m_mediaDecoder->GetNextFrame();
-				if (!m_mediaDecoder->SetPosition(m_time > 1.0 ? m_time - 1.0 : 0.0)) {
+				m_image = decoder->GetNextFrame();
+				if (!decoder->SetPosition(m_time > 1.0 ? m_time - 1.0 : 0.0)) {
 					wxLog* oldLog = wxLog::SetActiveTarget(new wxLogStderr());
 					wxLogError(wxT("decoder.GetDuration(): %f"), m_duration);
 					wxLogError(wxT("decoder.SetPosition(%f) failed"), m_time > 1.0 ? m_time - 1.0 : m_time);
 					delete wxLog::SetActiveTarget(oldLog);
 				}
 				for (int i = 0; i < 60; i++) {
-					m_image = m_mediaDecoder->GetNextFrame();
-					double currTime = m_mediaDecoder->GetPosition();
+					m_image = decoder->GetNextFrame();
+					double currTime = decoder->GetPosition();
 					if (currTime >= m_time || currTime < 0)
 						break;
 				}
 			} else
-				m_image = m_mediaDecoder->GetNextFrame();
+				m_image = decoder->GetNextFrame();
+			m_videoData = new wxSVGCanvasVideoData(decoder);
 		} else {
-			delete m_mediaDecoder;
-			m_mediaDecoder = NULL;
+			delete decoder;
 			m_duration = 0;
 		}
-#endif
 	}
+#endif
 }
