@@ -3,7 +3,7 @@
 // Purpose:     Agg render
 // Author:      Alex Thuering
 // Created:     2005/05/04
-// RCS-ID:      $Id: SVGCanvasAgg.cpp,v 1.15 2010/08/29 16:56:11 ntalex Exp $
+// RCS-ID:      $Id: SVGCanvasAgg.cpp,v 1.17 2011/06/27 21:14:14 ntalex Exp $
 // Copyright:   (c) 2005 Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -28,23 +28,20 @@
 #include <agg_span_allocator.h>
 #include <agg_image_accessors.h>
 #include <agg_pixfmt_rgba.h>
+#include <agg_blur.h>
 
 	
-wxSVGCanvasAgg::wxSVGCanvasAgg(): m_rbuf(NULL), m_pixf(NULL),
-  m_rendererBase(NULL), m_rendererSolid(NULL)
-{
+wxSVGCanvasAgg::wxSVGCanvasAgg(): m_rbuf(NULL), m_pixf(NULL), m_rendererBase(NULL), m_rendererSolid(NULL) {
 }
 
-wxSVGCanvasAgg::~wxSVGCanvasAgg()
-{
+wxSVGCanvasAgg::~wxSVGCanvasAgg() {
   delete m_rbuf;
   delete m_pixf;
   delete m_rendererBase;
   delete m_rendererSolid;
 }
 
-void wxSVGCanvasAgg::Init(int width, int height)
-{
+void wxSVGCanvasAgg::Init(int width, int height, bool alpha) {
   m_image.Create(width, height);
   m_rbuf = new RenderingBuffer(m_image.GetData(), 
 	 m_image.GetWidth(), m_image.GetHeight(), m_image.GetWidth()*3);
@@ -53,27 +50,22 @@ void wxSVGCanvasAgg::Init(int width, int height)
   m_rendererSolid = new RendererSolid(*m_rendererBase);
 }
 
-wxImage wxSVGCanvasAgg::GetImage()
-{
+wxImage wxSVGCanvasAgg::GetImage() {
   return m_image;
 }
 
-void wxSVGCanvasAgg::Clear(wxRGBColor color)
-{
+void wxSVGCanvasAgg::Clear(wxRGBColor color) {
   if (!m_rendererBase)
     return;
   m_rendererBase->clear(agg::rgba(
     color.Red()/255, color.Green()/255, color.Blue()/255));
 }
 
-wxSVGCanvasPath* wxSVGCanvasAgg::CreateCanvasPath()
-{
+wxSVGCanvasPath* wxSVGCanvasAgg::CreateCanvasPath() {
   return new wxSVGCanvasPathAgg();
 }
 
-wxSVGCanvasItem* wxSVGCanvasAgg::CreateItem(wxSVGTextElement* element,
-  const wxCSSStyleDeclaration* style)
-{
+wxSVGCanvasItem* wxSVGCanvasAgg::CreateItem(wxSVGTextElement* element, const wxCSSStyleDeclaration* style) {
 #ifdef __WXMSW__
   wxSVGCanvasText* canvasText = new wxSVGCanvasTextAgg(this);
 #else
@@ -86,10 +78,8 @@ wxSVGCanvasItem* wxSVGCanvasAgg::CreateItem(wxSVGTextElement* element,
 }
 
 void wxSVGCanvasAgg::DrawItem(wxSVGCanvasItem& item, wxSVGMatrix& matrix,
-  const wxCSSStyleDeclaration& style, wxSVGSVGElement& svgElem)
-{
-  switch (item.GetType())
-  {
+  const wxCSSStyleDeclaration& style, wxSVGSVGElement& svgElem) {
+  switch (item.GetType())   {
 	case wxSVGCanvasItem::wxSVG_CANVAS_ITEM_PATH:
 	  DrawCanvasPath((wxSVGCanvasPathAgg&) item, matrix, style, svgElem);
 	  break;
@@ -108,8 +98,7 @@ void wxSVGCanvasAgg::DrawItem(wxSVGCanvasItem& item, wxSVGMatrix& matrix,
 }
 
 void wxSVGCanvasAgg::DrawCanvasPath(wxSVGCanvasPathAgg& canvasPath,
- wxSVGMatrix& matrix, const wxCSSStyleDeclaration& style, wxSVGSVGElement& svgElem)
-{
+ wxSVGMatrix& matrix, const wxCSSStyleDeclaration& style, wxSVGSVGElement& svgElem) {
   if (!m_rendererBase)
     return;
   rasterizer_scanline_aa<> ras;
@@ -120,8 +109,7 @@ void wxSVGCanvasAgg::DrawCanvasPath(wxSVGCanvasPathAgg& canvasPath,
   canvasPath.m_transform = trans_affine(matrix.GetA(), matrix.GetB(),
 	matrix.GetC(), matrix.GetD(), matrix.GetE(), matrix.GetF());
 	  
-  if (style.GetFill().Ok())
-  {
+  if (style.GetFill().Ok()) {
 	ras.reset();
 	ras.filling_rule(
 	  style.GetFillRule() == wxCSS_VALUE_EVENODD ? fill_even_odd : fill_non_zero);
@@ -137,8 +125,7 @@ void wxSVGCanvasAgg::DrawCanvasPath(wxSVGCanvasPathAgg& canvasPath,
       style.GetOpacity()*style.GetFillOpacity(), matrix, svgElem, canvasPath);
   }
 
-  if (style.GetStroke().Ok() && style.GetStrokeWidth()>0)
-  {
+  if (style.GetStroke().Ok() && style.GetStrokeWidth()>0) {
 	canvasPath.SetStrokeStyle(style);
     
 	ras.reset();
@@ -148,6 +135,28 @@ void wxSVGCanvasAgg::DrawCanvasPath(wxSVGCanvasPathAgg& canvasPath,
     PaintRasterizer(ras, style.GetStroke(),
       style.GetOpacity()*style.GetStrokeOpacity(), matrix, svgElem, canvasPath);
   }
+  
+//  if (style.GetFilter().GetCSSPrimitiveType() == wxCSS_URI && style.GetFilter().GetStringValue().length() > 1) {
+//	  wxSVGElement* filterElem = (wxSVGSVGElement*) svgElem.GetElementById(style.GetFilter().GetStringValue().substr(1));
+//	  // feGaussianBlur
+//	  if (filterElem && filterElem->GetDtd() == wxSVG_FILTER_ELEMENT && filterElem->GetFirstChild() != NULL
+//			  && ((wxSVGSVGElement*)filterElem->GetFirstChild())->GetDtd() == wxSVG_FEGAUSSIANBLUR_ELEMENT) {
+//		  float radius = ((wxSVGFEGaussianBlurElement*) filterElem->GetFirstChild())->GetStdDeviationX().GetAnimVal()*2;
+//		  // Calculate the bounding box and extend it by the blur radius
+//		  agg::rect_d bbox;
+//		  agg::bounding_rect_single(canvasPath.m_curved_trans, 0, &bbox.x1, &bbox.y1, &bbox.x2, &bbox.y2);
+//		  bbox.x1 -= radius;
+//		  bbox.y1 -= radius;
+//		  bbox.x2 += radius*2;
+//		  bbox.y2 += radius*2;
+//		  
+//		  PixFmt pixf2(*m_rbuf);
+//		  if(pixf2.attach(*m_pixf, int(bbox.x1), int(bbox.y1), int(bbox.x2), int(bbox.y2))) {
+//			  agg::recursive_blur<agg::rgba8, agg::recursive_blur_calc_rgb<> > recursive_blur;
+//			  recursive_blur.blur(pixf2, radius);
+//		  }
+//	  }
+//  }
 }
 
 
@@ -237,8 +246,8 @@ void wxSVGCanvasAgg::PaintRasterizer(rasterizer_scanline_aa<>& ras,
   if (paint.GetPaintType() >= wxSVG_PAINTTYPE_URI_NONE &&
       paint.GetPaintType() <= wxSVG_PAINTTYPE_URI)
   {
-    const wxSVGElement* refElem;
-    int m_nstops = GetGradientStops(svgElem, paint.GetUri(), opacity, refElem);
+	wxSVGGradientElement* gradElem = GetGradientElement(svgElem, paint.GetUri());
+    int m_nstops = GetGradientStops(svgElem, gradElem, opacity);
     if (m_nstops)
     {
       for (unsigned int i = int(m_offset * 255); i <= 255; i++)
@@ -255,13 +264,12 @@ void wxSVGCanvasAgg::PaintRasterizer(rasterizer_scanline_aa<>& ras,
       
       wxSVGMatrix rgMatrix = matrix;
       double length = 0;
-      switch (refElem->GetDtd())
+      switch (gradElem->GetDtd())
       {
         case wxSVG_LINEARGRADIENT_ELEMENT:
         {
-          wxSVGLinearGradientElement* gradElem = (wxSVGLinearGradientElement*)refElem;
           wxSVGPoint p1, p2;
-          GetLinearGradientVector(p1, p2, *gradElem, path);
+          GetLinearGradientVector(p1, p2, (wxSVGLinearGradientElement&) *gradElem, path);
           double dx = p2.GetX() - p1.GetX();
           double dy = p2.GetY() - p1.GetY();
           rgMatrix = rgMatrix.Translate(p1.GetX(), p1.GetY());
@@ -272,9 +280,8 @@ void wxSVGCanvasAgg::PaintRasterizer(rasterizer_scanline_aa<>& ras,
         case wxSVG_RADIALGRADIENT_ELEMENT:
         {
           gr_ptr = &gr_circle;
-          wxSVGRadialGradientElement* gradElem = (wxSVGRadialGradientElement*)refElem;
           wxSVGPoint focus;
-          GetRadialGradientTransform(focus, rgMatrix, *gradElem, path);
+          GetRadialGradientTransform(focus, rgMatrix, (wxSVGRadialGradientElement&) *gradElem, path);
           length = gradElem->GetR().GetAnimVal();
           break;
         }
